/******************************
 * classe qui simule le comportement d'une balle
 * dans un monde imaginaire où la gravité s'exerce dans 2 directions orthogonales
 *******************************************/
class Balle {
// --------- les champs de la classe ------------

  // rayon de la balle
  int r;
  
  // coefficient de friction
  float friction;
  
  // position horizontale et verticale
  float x, y;
  
  // vitesse horizontale et verticale
  float vx, vy;
  
// ---------- les constructeurs ------------
  
  /****************************************
   * constructeur qui initialise tous les champs
   *
   * entrée : rInit, fInit, yInit, vyInit : valeur intiales
   ****************************************/
  Balle(int rInit, float fInit, float xInit, float yInit, float vxInit, float vyInit) {
    r        = rInit;
    friction = fInit;
    y        = yInit;
    x        = xInit;
    vx       = vxInit;
    vy       = vyInit;
  }
  
// ---------- les méthodes ------------

  /****************************************
   * permet le dessin sur le canevas de la balle
   *
   * entrée : 
   *     aucune
   *
   * sortie :
   *     aucune seulement changement du canevas
   ****************************************/
  void dessineToi() {
    ellipse(x, y, 2 * r, 2 * r);
  }
  
  /****************************************
   * modifie la position et la vitesse de la balle
   *
   * entrée : 
   *     aucune
   *
   * sortie :
   *     aucune seulement changement de l'état interne
   ****************************************/
  void bouge() {
    x = x + vx;
    y = y + vy;
    
    if (x + r > width) {
      x = width - r;
      vx = - vx;
      vx = vx * friction ;
    }
    
    if (y + r > height) {
      y = height - r;
      vy = - vy;
      vy = vy * friction ;
    }
    
    // la gravité est deux fois moins grande horizontalement
    vx = vx + 0.5;
    vy = vy + 1;
  }
  
  /****************************************
   * donne l'état de la balle
   *
   * entrée : 
   *     aucune
   *
   * sortie :
   *     chaine de caractères donnant l'état
   ****************************************/
  public String toString() {
    return "Balle [r=" + r + ", friction=" + friction + ", x=" + x + ", y=" + y + ", vx=" + vx + ", vy=" + vy + "]" ;
  }
  
}
 
/**************************************************/
Balle b;

void setup() {
  size(400, 400);
  fill(0, 0, 255);
  noStroke();
  b = new Balle(10, 0.9, 0, 0, 0, 0);
  println("Balle initiale : " + b);
}

void draw() {
  background(255, 255, 0);
  b.dessineToi();
  b.bouge();
}
